@@ -1,7 +1,5 @@
 /*  
-    Jasjeet S. Sekhon <sekhon@berkeley.edu>
-    HTTP://sekhon.berkeley.edu/
-    UC Berkeley
+    Jasjeet S. Sekhon <jasjeet.sekhon@gmail.com>
     
     2013/10/28
     Under the GNU Public License Version 3
@@ -78,6 +76,13 @@
 /* Note: We are using the include cblas header which will direct to the
    central R BLAS */
 
+#define USE_FC_LEN_T
+#include <Rconfig.h>
+#include <R_ext/BLAS.h>
+#ifndef FCONE
+# define FCONE
+#endif
+
 #include "scythematrix.h"
 
 using namespace SCYTHE;
@@ -114,38 +119,12 @@ using namespace std;
 
 extern "C"
 {
-#ifdef INTERNAL_CBLAS
-#include "cblas.h"
-
-  void cblas_dgemm(const enum CBLAS_ORDER Order, const enum CBLAS_TRANSPOSE TransA,
-		   const enum CBLAS_TRANSPOSE TransB, const int M, const int N,
-		   const int K, const double alpha, const double  *A,
-		   const int lda, const double  *B, const int ldb,
-		   const double beta, double  *C, const int ldc);
-
-  void cblas_dscal( const int N, const double alpha, double *X, 
-		    const int incX);
-
-  double cblas_dasum( const int N, const double *X, const int incX);
-#endif
-
 /*---------------------------------------------------------------------------
    Function :   kth_smallest()
    In       :   array of elements, # of elements in the array, rank k
    Out      :   one element
    Job      :   find the kth smallest element in the array
    Notice   :   use the median() macro defined below to get the median. 
-
-                Reference:
-
-                http://www.eso.org/~ndevilla/median/
-
-                  Author: Wirth, Niklaus 
-                   Title: Algorithms + data structures = programs 
-               Publisher: Englewood Cliffs: Prentice-Hall, 1976 
-    Physical description: 366 p. 
-                  Series: Prentice-Hall Series in Automatic Computation 
-
  ---------------------------------------------------------------------------*/
 
 
@@ -485,7 +464,7 @@ extern "C"
 	//http://docs.sun.com/source/819-3691/dscal.html
 	for (int kk=0; kk < xvars; kk++)
 	  {
-	    cblas_dscal(N, ww.data[M(kk,kk,xvars)], ZX.data+kk, xvars);
+	    F77_dscal(N, ww.data[M(kk,kk,xvars)], ZX.data+kk, xvars);
 	  }
 
 	ZX.multi_scalar(ZX);
@@ -839,7 +818,7 @@ extern "C"
 	//http://docs.sun.com/source/819-3691/dscal.html
 	for (int kk=0; kk < xvars; kk++)
 	  {
-	    cblas_dscal(N, ww.data[M(kk,kk,xvars)], ZX.data+kk, xvars);
+	    F77_dscal(N, ww.data[M(kk,kk,xvars)], ZX.data+kk, xvars);
 	  }
 
 	ZX.multi_scalar(ZX);
@@ -1176,9 +1155,10 @@ extern "C"
     } /* if (restrict_trigger==1) */
 
     // required for replace=0, to keep track of obs already used
-    long *ReplaceVector;
+    long *ReplaceVector = (long *) malloc(sizeof(long));
     if(replace==0)
       {
+  free(ReplaceVector);    
 	ReplaceVector = (long *) malloc(N*sizeof(long));
       }
 
@@ -1246,21 +1226,25 @@ extern "C"
 		//JSS
 		// do the second multiplication with dgemm, multiplying the matrix
 		// above, D, by the transpose of matrix W.
-		cblas_dgemm(CblasRowMajor,// column major
-			    CblasNoTrans, // A not transposed
-			    CblasTrans,   // B transposed
-			    xvars,        // M
-			    N,            // N
-			    xvars,        // K
-			    1.0,          // alpha, (alpha * A * B)
-			    ww.data,      // A
-			    xvars,        // leading dimension for A
-			    ZX.data,      // B
-			    xvars,        // leading dimension for B
-			    0.0,          // beta, (beta * C)
-			    DX.data,      // C
-			    N);           // leading dimension for C
-		
+		const double one=1.0;
+		const double zero=0.0;
+		const int fxvars= (int) xvars;
+		const int fN= (int) N;
+		F77_CALL(dgemm)(
+				"T",   // A not transposed (do the opposite because of RowMajor)
+				"N",   // B transposed (do the opposite because of RowMajor)
+				&fN,            // N
+				&fxvars,        // M
+				&fxvars,        // K
+				&one,           // alpha, (alpha * A * B)
+				ZX.data,        // B
+				&fxvars,        // leading dimension for B, ldb				
+				ww.data,        // A
+				&fxvars,        // leading dimension for A, lda				
+				&zero,          // beta, (beta * C)
+				DX.data,        // C				
+				&fN FCONE FCONE);  // leading dimension for C
+
 		DX.multi_scalar(DX);
 		
 		std::swap(DX.colsize, DX.rowsize);
@@ -1286,7 +1270,7 @@ extern "C"
 		//http://docs.sun.com/source/819-3691/dscal.html
 		for (int kk=0; kk < xvars; kk++)
 		  {
-		    cblas_dscal(N, ww.data[M(kk,kk,xvars)], ZX.data+kk, xvars);
+		    F77_dscal(N, ww.data[M(kk,kk,xvars)], ZX.data+kk, xvars);
 		  }
 		
 		ZX.multi_scalar(ZX);
@@ -1344,7 +1328,7 @@ extern "C"
 		      {
 			
 			if (restrict[M(j,2,3)] < 0) {
-			  Dist[ ((long) restrict[M(j,1,3)])-1 ] = DOUBLE_XMAX;
+			  Dist[ ((long) restrict[M(j,1,3)])-1 ] = DBL_MAX;
 			}
 			else {
 			  Dist[ ((long) restrict[M(j,1,3)])-1 ] = restrict[M(j,2,3)];
@@ -1354,7 +1338,7 @@ extern "C"
 		      {
 			
 			if (restrict[M(j,2,3)] < 0) {
-			  Dist[ ((long) restrict[M(j,0,3)])-1 ] = DOUBLE_XMAX;
+			  Dist[ ((long) restrict[M(j,0,3)])-1 ] = DBL_MAX;
 			}
 			else {
 			  Dist[ ((long) restrict[M(j,0,3)])-1 ] = restrict[M(j,2,3)];
@@ -1368,7 +1352,7 @@ extern "C"
 	      {
 		for(j=0; j<replace_count; j++)
 		  {
-		    Dist[ ( ReplaceVector[j] - 1) ] = DOUBLE_XMAX;
+		    Dist[ ( ReplaceVector[j] - 1) ] = DBL_MAX;
 		  }
 	      } // end of replace_count
 
@@ -1387,7 +1371,7 @@ extern "C"
 			    diff = abs(Xorig[M(i, k, xvars)] - Xorig[M(j,k,xvars)]); 
 			    if (diff > CaliperVec[k])
 			      {
-				Dist[j] = DOUBLE_XMAX;
+				Dist[j] = DBL_MAX;
 				break;
 			      }
 			  }
@@ -1429,7 +1413,7 @@ extern "C"
 
 	    if ( (restrict_trigger==1) | (caliper==1) )
 	      {
-		if ( (Distmax+cdd) > DOUBLE_XMAX_CHECK)
+		if ( (Distmax+cdd) > DBL_MAX_CHECK)
 		  {
 		    sum_caliper_drops++;
 		    continue;
@@ -1592,10 +1576,7 @@ extern "C"
     /* Free Memory */
     free(S);
     free(order_DistPot);
-    if(replace==0)
-      {
-	free(ReplaceVector);
-      }
+	  free(ReplaceVector);
     
     r = rows(rr);
     c = cols(rr);
@@ -1778,9 +1759,10 @@ extern "C"
     } /* if (restrict_trigger==1) */
 
     // required for replace=0, to keep track of obs already used
-    long *ReplaceVector;
+    long *ReplaceVector = (long *) malloc(sizeof(long));
     if(replace==0)
       {
+  free(ReplaceVector);    
 	ReplaceVector = (long *) malloc(N*sizeof(long));
       }
 
@@ -1857,20 +1839,24 @@ extern "C"
 		//JSS
 		// do the second multiplication with dgemm, multiplying the matrix
 		// above, D, by the transpose of matrix W.
-		cblas_dgemm(CblasRowMajor,// column major
-			    CblasNoTrans, // A not transposed
-			    CblasTrans,   // B transposed
-			    xvars,        // M
-			    N,            // N
-			    xvars,        // K
-			    1.0,          // alpha, (alpha * A * B)
-			    ww.data,      // A
-			    xvars,        // leading dimension for A
-			    ZX.data,      // B
-			    xvars,        // leading dimension for B
-			    0.0,          // beta, (beta * C)
-			    DX.data,      // C
-			    N);           // leading dimension for C
+		const double one=1.0;
+		const double zero=0.0;
+		const int fxvars= (int) xvars;
+		const int fN= (int) N;
+		F77_CALL(dgemm)(
+				"T",   // A not transposed (do the opposite because of RowMajor)
+				"N",   // B transposed (do the opposite because of RowMajor)
+				&fN,            // N
+				&fxvars,        // M
+				&fxvars,        // K
+				&one,           // alpha, (alpha * A * B)
+				ZX.data,        // B
+				&fxvars,        // leading dimension for B, ldb				
+				ww.data,        // A
+				&fxvars,        // leading dimension for A, lda				
+				&zero,          // beta, (beta * C)
+				DX.data,        // C				
+				&fN FCONE FCONE);  // leading dimension for C
 		
 		DX.multi_scalar(DX);
 		
@@ -1897,7 +1883,7 @@ extern "C"
 		//http://docs.sun.com/source/819-3691/dscal.html
 		for (int kk=0; kk < xvars; kk++)
 		  {
-		    cblas_dscal(N, ww.data[M(kk,kk,xvars)], ZX.data+kk, xvars);
+		    F77_dscal(N, ww.data[M(kk,kk,xvars)], ZX.data+kk, xvars);
 		  }
 		
 		ZX.multi_scalar(ZX);
@@ -1955,7 +1941,7 @@ extern "C"
 		      {
 			
 			if (restrict[M(j,2,3)] < 0) {
-			  Dist[ ((long) restrict[M(j,1,3)])-1 ] = DOUBLE_XMAX;
+			  Dist[ ((long) restrict[M(j,1,3)])-1 ] = DBL_MAX;
 			}
 			else {
 			  Dist[ ((long) restrict[M(j,1,3)])-1 ] = restrict[M(j,2,3)];
@@ -1965,7 +1951,7 @@ extern "C"
 		      {
 			
 			if (restrict[M(j,2,3)] < 0) {
-			  Dist[ ((long) restrict[M(j,0,3)])-1 ] = DOUBLE_XMAX;
+			  Dist[ ((long) restrict[M(j,0,3)])-1 ] = DBL_MAX;
 			}
 			else {
 			  Dist[ ((long) restrict[M(j,0,3)])-1 ] = restrict[M(j,2,3)];
@@ -1979,7 +1965,7 @@ extern "C"
 	      {
 		for(j=0; j<replace_count; j++)
 		  {
-		    Dist[ ( ReplaceVector[j] - 1) ] = DOUBLE_XMAX;
+		    Dist[ ( ReplaceVector[j] - 1) ] = DBL_MAX;
 		  }
 	      } // end of replace_count
 
@@ -1998,7 +1984,7 @@ extern "C"
 			    diff = abs(Xorig[M(i, k, xvars)] - Xorig[M(j,k,xvars)]); 
 			    if (diff > CaliperVec[k])
 			      {
-				Dist[j] = DOUBLE_XMAX;
+				Dist[j] = DBL_MAX;
 				break;
 			      }
 			  }
@@ -2019,7 +2005,7 @@ extern "C"
 
 	    if ( (restrict_trigger==1) | (caliper==1) )
 	      {
-		if ( (Distmax+cdd) > DOUBLE_XMAX_CHECK)
+		if ( (Distmax+cdd) > DBL_MAX_CHECK)
 		  {
 		    sum_caliper_drops++;
 		    continue;
@@ -2217,10 +2203,7 @@ extern "C"
     /* Free Memory */
     //free(S);
     free(order_DistPot);
-    if(replace==0)
-      {
-	free(ReplaceVector);
-      }
+	  free(ReplaceVector);
     
     r = rows(rr);
     c = cols(rr);
@@ -2465,20 +2448,24 @@ extern "C"
 	    //JSS
 	    // do the second multiplication with dgemm, multiplying the matrix
 	    // above, D, by the transpose of matrix W.
-	    cblas_dgemm(CblasRowMajor,// column major
-			CblasNoTrans, // A not transposed
-			CblasTrans,   // B transposed
-			xvars,        // M
-			N,            // N
-			xvars,        // K
-			1.0,          // alpha, (alpha * A * B)
-			ww.data,      // A
-			xvars,        // leading dimension for A
-			ZX.data,      // B
-			xvars,        // leading dimension for B
-			0.0,          // beta, (beta * C)
-			DX.data,      // C
-			N);           // leading dimension for C
+	    const double one=1.0;
+	    const double zero=0.0;
+	    const int fxvars= (int) xvars;
+	    const int fN= (int) N;
+	    F77_CALL(dgemm)(
+			    "T",   // A not transposed (do the opposite because of RowMajor)
+			    "N",   // B transposed (do the opposite because of RowMajor)
+			    &fN,            // N
+			    &fxvars,        // M
+			    &fxvars,        // K
+			    &one,           // alpha, (alpha * A * B)
+			    ZX.data,        // B
+			    &fxvars,        // leading dimension for B, ldb				
+			    ww.data,        // A
+			    &fxvars,        // leading dimension for A, lda				
+			    &zero,          // beta, (beta * C)
+			    DX.data,        // C				
+			    &fN FCONE FCONE);  // leading dimension for C
 	    
 	    DX.multi_scalar(DX);
 	    
@@ -2505,7 +2492,7 @@ extern "C"
 	    //http://docs.sun.com/source/819-3691/dscal.html
 	    for (int kk=0; kk < xvars; kk++)
 	      {
-		cblas_dscal(N, ww.data[M(kk,kk,xvars)], ZX.data+kk, xvars);
+		F77_dscal(N, ww.data[M(kk,kk,xvars)], ZX.data+kk, xvars);
 	      }
 	    
 	    ZX.multi_scalar(ZX);
@@ -2553,7 +2540,7 @@ extern "C"
 #endif /* end __NBLAS__ */
 
 	//Remove self as a potential match
-	Dist[i] = DOUBLE_XMAX;
+	Dist[i] = DBL_MAX;
 	
 	// Dist distance to observation to be matched
 	// is N by 1 vector	    
@@ -2565,7 +2552,7 @@ extern "C"
 		  {
 		    
 		    if (restrict[M(j,2,3)] < 0) {
-		      Dist[ ((long) restrict[M(j,1,3)])-1 ] = DOUBLE_XMAX;
+		      Dist[ ((long) restrict[M(j,1,3)])-1 ] = DBL_MAX;
 		    }
 		    else {
 		      Dist[ ((long) restrict[M(j,1,3)])-1 ] = restrict[M(j,2,3)];
@@ -2575,7 +2562,7 @@ extern "C"
 		  {
 		    
 		    if (restrict[M(j,2,3)] < 0) {
-		      Dist[ ((long) restrict[M(j,0,3)])-1 ] = DOUBLE_XMAX;
+		      Dist[ ((long) restrict[M(j,0,3)])-1 ] = DBL_MAX;
 		    }
 		    else {
 		      Dist[ ((long) restrict[M(j,0,3)])-1 ] = restrict[M(j,2,3)];
@@ -2598,7 +2585,7 @@ extern "C"
 			diff = abs(Xorig[M(i, k, xvars)] - Xorig[M(j,k,xvars)]); 
 			if (diff > CaliperVec[k])
 			  {
-			    Dist[j] = DOUBLE_XMAX;
+			    Dist[j] = DBL_MAX;
 			    break;
 			  }
 		      }
@@ -2616,7 +2603,7 @@ extern "C"
 	    
 	    if ( (restrict_trigger==1) | (caliper==1) )
 	      {
-		if ( (Distmax+cdd) > DOUBLE_XMAX_CHECK)
+		if ( (Distmax+cdd) > DBL_MAX_CHECK)
 		  {
 		    sum_caliper_drops++;
 		    continue;
@@ -2684,7 +2671,7 @@ extern "C"
 	  
 	  if ( (restrict_trigger==1) | (caliper==1) )
 	    {
-	      if ( (Distmax+cdd) > DOUBLE_XMAX_CHECK)
+	      if ( (Distmax+cdd) > DBL_MAX_CHECK)
 		{
 		  sum_caliper_drops++;
 		  continue;
@@ -2806,7 +2793,7 @@ Matrix EqualityTestMatrix(Matrix a, Matrix s)
 	{
 	  for (long j =0; j< ncols; j++)
 	    {
-	      if( (a[M(i, j, ncols)] < (s[i]+TOL)) &  (a[M(i, j, ncols)] > (s[i]-TOL)) )
+	      if( (a[M(i, j, ncols)] < (s[i]+TOL)) &&  (a[M(i, j, ncols)] > (s[i]-TOL)) )
 		{
 		  ret[M(i, j, ncols)] = 1;
 		}
@@ -2819,7 +2806,7 @@ Matrix EqualityTestMatrix(Matrix a, Matrix s)
 	{
 	  for (long j =0; j< ncols; j++)
 	    {
-	      if( (a[M(i, j, ncols)] < (s[M(i, j, ncols)]+TOL)) &  
+	      if( (a[M(i, j, ncols)] < (s[M(i, j, ncols)]+TOL)) &&  
 		  (a[M(i, j, ncols)] > (s[M(i, j, ncols)]-TOL)) )
 		{
 		  ret[M(i, j, ncols)] = 1;
@@ -2898,3 +2885,17 @@ Matrix multi_scalar (Matrix a, Matrix b)
   a.multi_scalar(b);
   return a;
 } 
+
+
+void F77_dscal( const int N, const double DA, double *DX, 
+                       const int INCX)
+{
+  F77_CALL(dscal)( &N, &DA, DX, &INCX);
+}
+
+void F77_dasum( const int N, const double *DX, const int INCX)
+{
+  F77_CALL(dasum)( &N, DX, &INCX);
+}
+
+
